/*
 * ---- 
 *
 * $Id: ChannelSearchProvider.java,v 1.3 2002/02/19 08:56:47 burton Exp $
 * $Project: http://reptile.openprivacy.org $
 * $CVSROOT: :pserver:anoncvs@sierra.openprivacy.org:/usr/local/cvs/public $
 * $WebCVS: http://www.openprivacy.org/cgi-bin/cvsweb/cvsweb.cgi/sierra/ $
 * $Mailing-List: http://www.openprivacy.org/lists/ $
 * $Bugzilla: http://bugzilla.openprivacy.org/ $
 *
 * ----
 *
 * Copyright 2001 OpenPrivacy.org  All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the LICENSE which you should have obtaind with this package. 
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.
 */

package org.openprivacy.reptile.search.impl;

import java.io.*;
import java.net.*;
import java.util.*;

import org.openprivacy.reptile.actions.*;
import org.openprivacy.reptile.extensions.*;
import org.openprivacy.reptile.init.*;
import org.openprivacy.reptile.om.*;
import org.openprivacy.reptile.search.*;
import org.openprivacy.reptile.util.*;
import org.openprivacy.reptile.xslt.*;

// Village classes
import com.workingdogs.village.*;

// Turbine classes
import org.apache.turbine.om.peer.*;
import org.apache.turbine.util.*;
import org.apache.turbine.util.db.*;
import org.apache.turbine.util.db.map.*;
import org.apache.turbine.util.db.pool.DBConnection;

// Local classes
import org.openprivacy.reptile.om.map.*;

/**
 * Handles searching articles in the OM database article store.
 * 
 * @see SearchProvider
 * @see AbstractSearchProvider
 * @author <a href="mailto:burton@openprivacy.org">Kevin A. Burton</a>
 * @version $Id: ChannelSearchProvider.java,v 1.3 2002/02/19 08:56:47 burton Exp $
 */
public class ChannelSearchProvider extends DBSearchProvider {

    /**
     * 
     * Create a new <code>ChannelSearchProvider</code> instance.
     *
     * @author <a href="mailto:burton@openprivacy.org">Kevin A. Burton</a>
     */
    public ChannelSearchProvider() {
        
        super( ChannelPeer.class );

        super.defaultSearchFields.add( ChannelPeer.TITLE );
        super.defaultSearchFields.add( ChannelPeer.DESCRIPTION );

        super.defaultSortOrder.add( ChannelPeer.DATE_FOUND );
        
    }
    

    /**
     * @see SearchProvider#record
     * @throws Exception When an invalid SearchRecord index is used
     * @author <a href="mailto:burton@openprivacy.org">Kevin A. Burton</a>
     */
    public void record( SearchRecord record ) throws Exception {

        record.assertValid( this );
        
        int index = record.getIndex();

        Channel channel = (Channel)getResults().elementAt( index );

        record.setTitle( channel.getTitle() );
        record.setDateFound( channel.getDateFound() );
        record.setDescription( channel.getDescription() );
        record.setLink( channel.getLocation() );
        
        record.setChannel( channel.getLocation() );
        
    }

    /**
     * 
     * @see SearchProvider#destroy
     * @author <a href="mailto:burton@openprivacy.org">Kevin A. Burton</a>
     */
    public void destroy() {}
    
}
